import * as path from "path";
import { UndefinedComponent } from "../../../Exception";
import * as Guard from "../Guard";
import * as Name from "../Name";
import * as MediaType from "./MediaType";
import * as Reference from "./Reference";
import * as Response from "./Response";
export const generateNamespace = (entryPoint, currentPoint, store, factory, responses, context) => {
    const basePath = "components/responses";
    store.addComponent("responses", {
        type: "namespace",
        name: Name.Components.Responses,
        value: factory.Namespace.create({
            export: true,
            name: Name.Components.Responses,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#responsesObject`,
        }),
        statements: {},
    });
    Object.entries(responses).forEach(([name, response]) => {
        if (Guard.isReference(response)) {
            const reference = Reference.generate(entryPoint, currentPoint, response);
            if (reference.type === "local") {
                if (!store.hasStatement(reference.path, ["interface"])) {
                    throw new UndefinedComponent(`Reference "${response.$ref}" did not found in ${reference.path} by ${reference.name}`);
                }
            }
            else if (reference.type === "remote") {
                Response.generateNamespace(entryPoint, reference.referencePoint, store, factory, path.dirname(reference.path), // referencePoint basename === namespace name
                reference.name, reference.data, context);
            }
        }
        else {
            Response.generateNamespace(entryPoint, currentPoint, store, factory, basePath, name, response, context);
        }
    });
};
export const generateNamespaceWithStatusCode = (entryPoint, currentPoint, store, factory, parentPath, responses, context) => {
    const basePath = `${parentPath}/responses`;
    store.addStatement(basePath, {
        type: "namespace",
        name: Name.ComponentChild.Response,
        value: factory.Namespace.create({
            export: true,
            name: Name.ComponentChild.Response,
            statements: [],
            comment: `@see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#responsesObject`,
        }),
        statements: {},
    });
    Object.entries(responses).forEach(([statusCode, response]) => {
        const nameWithStatusCode = `Status$${statusCode}`;
        if (Guard.isReference(response)) {
            const reference = Reference.generate(entryPoint, currentPoint, response);
            if (reference.type === "local") {
                context.setReferenceHandler(reference);
                Response.generateReferenceNamespace(entryPoint, currentPoint, store, factory, basePath, nameWithStatusCode, reference, context);
            }
            else if (reference.componentName) {
                // reference先に定義を作成
                Response.generateNamespace(entryPoint, reference.referencePoint, store, factory, path.dirname(reference.path), // referencePoint basename === namespace name
                reference.name, reference.data, context);
                // referenceのTypeAliasの作成
                Response.generateReferenceNamespace(entryPoint, currentPoint, store, factory, basePath, nameWithStatusCode, reference, context);
            }
        }
        else {
            Response.generateNamespace(entryPoint, currentPoint, store, factory, basePath, nameWithStatusCode, response, context);
        }
    });
};
export const generateInterfacesWithStatusCode = (entryPoint, currentPoint, store, factory, operationId, responses, context) => {
    const statements = [];
    Object.entries(responses).forEach(([statusCode, response]) => {
        if (!response) {
            return;
        }
        if (Guard.isReference(response)) {
            const reference = Reference.generate(entryPoint, currentPoint, response);
            if (reference.type === "local") {
                context.setReferenceHandler(reference);
                statements.push(factory.TypeAliasDeclaration.create({
                    export: true,
                    name: Name.responseName(operationId, statusCode),
                    type: factory.TypeReferenceNode.create({
                        name: context.getReferenceName(currentPoint, `${reference.path}/Content`),
                    }),
                }));
            }
            else if (reference.componentName) {
                // reference先に定義を作成
                Response.generateNamespace(entryPoint, reference.referencePoint, store, factory, path.dirname(reference.path), // referencePoint basename === namespace name
                reference.name, reference.data, context);
                // referenceのTypeAliasの作成
                const content = reference.data.content;
                if (content) {
                    statements.push(MediaType.generateInterface(entryPoint, reference.referencePoint, factory, Name.responseName(operationId, statusCode), content, context));
                }
            }
        }
        else {
            if (response.content) {
                statements.push(MediaType.generateInterface(entryPoint, currentPoint, factory, Name.responseName(operationId, statusCode), response.content, context));
            }
        }
    });
    return statements;
};
//# sourceMappingURL=Responses.js.map