import { FeatureDevelopmentError } from "../../../Exception";
import * as Guard from "../Guard";
import * as ToTypeNode from "../toTypeNode";
import * as ExternalDocumentation from "./ExternalDocumentation";
export const generatePropertySignatures = (entryPoint, currentPoint, factory, schema, context) => {
    if (!schema.properties) {
        return [];
    }
    const required = schema.required || [];
    return Object.entries(schema.properties).map(([propertyName, property]) => {
        return factory.PropertySignature.create({
            name: propertyName,
            optional: !required.includes(propertyName),
            type: ToTypeNode.convert(entryPoint, currentPoint, factory, property, context, { parent: schema }),
            comment: typeof property !== "boolean" ? property.description : undefined,
        });
    });
};
export const generateInterface = (entryPoint, currentPoint, factory, name, schema, context) => {
    if (schema.type !== "object") {
        throw new FeatureDevelopmentError("Please use generateTypeAlias");
    }
    let members = [];
    const propertySignatures = generatePropertySignatures(entryPoint, currentPoint, factory, schema, context);
    if (Guard.isObjectSchemaWithAdditionalProperties(schema)) {
        const additionalProperties = ToTypeNode.convertAdditionalProperties(entryPoint, currentPoint, factory, schema, context);
        if (schema.additionalProperties === true) {
            members = members.concat(additionalProperties);
        }
        else {
            members = [...propertySignatures, additionalProperties];
        }
    }
    else {
        members = propertySignatures;
    }
    return factory.InterfaceDeclaration.create({
        export: true,
        name,
        members,
        comment: ExternalDocumentation.addComment(schema.description, schema.externalDocs),
    });
};
export const generateArrayTypeAlias = (entryPoint, currentPoint, factory, name, schema, context) => {
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        comment: schema.description,
        type: ToTypeNode.convert(entryPoint, currentPoint, factory, schema, context),
    });
};
export const generateTypeAlias = (entryPoint, currentPoint, factory, name, schema) => {
    let type;
    if (schema.enum) {
        if (Guard.isNumberArray(schema.enum) && (schema.type === "number" || schema.type === "integer")) {
            type = factory.TypeNode.create({
                type: schema.type,
                enum: schema.enum,
            });
        }
        else if (Guard.isStringArray(schema.enum) && schema.type === "string") {
            type = factory.TypeNode.create({
                type: schema.type,
                enum: schema.enum,
            });
        }
        else {
            type = factory.TypeNode.create({
                type: schema.type,
            });
        }
    }
    else {
        type = factory.TypeNode.create({
            type: schema.type,
        });
    }
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        type,
    });
};
export const generateMultiTypeAlias = (entryPoint, currentPoint, factory, name, schemas, context, multiType) => {
    const type = ToTypeNode.generateMultiTypeNode(entryPoint, currentPoint, factory, schemas, context, ToTypeNode.convert, multiType);
    return factory.TypeAliasDeclaration.create({
        export: true,
        name,
        type,
    });
};
export const addSchema = (entryPoint, currentPoint, store, factory, targetPoint, declarationName, schema, context) => {
    if (!schema) {
        return;
    }
    if (Guard.isAllOfSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: generateMultiTypeAlias(entryPoint, currentPoint, factory, declarationName, schema.allOf, context, "allOf"),
        });
    }
    else if (Guard.isOneOfSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: generateMultiTypeAlias(entryPoint, currentPoint, factory, declarationName, schema.oneOf, context, "oneOf"),
        });
    }
    else if (Guard.isAnyOfSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: generateMultiTypeAlias(entryPoint, currentPoint, factory, declarationName, schema.anyOf, context, "allOf"),
        });
    }
    else if (Guard.isArraySchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: generateArrayTypeAlias(entryPoint, currentPoint, factory, declarationName, schema, context),
        });
    }
    else if (Guard.isObjectSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "interface",
            name: declarationName,
            value: generateInterface(entryPoint, currentPoint, factory, declarationName, schema, context),
        });
    }
    else if (Guard.isPrimitiveSchema(schema)) {
        store.addStatement(targetPoint, {
            type: "typeAlias",
            name: declarationName,
            value: generateTypeAlias(entryPoint, currentPoint, factory, declarationName, schema),
        });
    }
};
//# sourceMappingURL=Schema.js.map