import * as fs from "fs";
import { relative } from "path";
import Dot from "dot-prop";
import yaml from "js-yaml";
import { UnSupportError } from "../../../Exception";
import * as Def from "./Definition";
import * as Masking from "./masking";
import * as Operation from "./Operation";
import * as PropAccess from "./PropAccess";
import * as State from "./State";
export const create = (factory, rootDocument) => {
    const state = State.createDefaultState(rootDocument);
    const getPathItem = (localPath) => {
        if (!localPath.startsWith("components/pathItem")) {
            throw new Error("Only use start with 'component/pathItems': " + localPath);
        }
        const result = Dot.get(state.document, localPath.replace(/\//g, "."));
        if (!result) {
            throw new Error(`Not found ${localPath}`);
        }
        return result;
    };
    const createNamespace = (name) => {
        const value = factory.Namespace.create({
            export: true,
            name,
            statements: [],
        });
        return {
            type: "namespace",
            name,
            value,
            statements: {},
        };
    };
    const addComponent = (componentName, statement) => {
        const key = Def.generateKey("namespace", componentName);
        state.components[key] = statement;
    };
    const getStatement = (path, type) => {
        const targetPath = relative("components", path);
        const result = PropAccess.get(state.components, type, targetPath);
        return result;
    };
    const hasStatement = (path, types) => {
        const targetPath = relative("components", path);
        return types.some(type => !!PropAccess.get(state.components, type, targetPath));
    };
    const addStatement = (path, statement) => {
        if (!path.startsWith("components")) {
            throw new UnSupportError(`componentsから始まっていません。path=${path}`);
        }
        const targetPath = relative("components", path);
        state.components = PropAccess.set(state.components, targetPath, statement, createNamespace);
    };
    const interfaceToStatement = (node) => {
        return node.value;
    };
    const typeAliasToStatement = (node) => {
        return node.value;
    };
    const namespaceToTsStatement = (node) => {
        const statements = Object.values(node.statements).reduce((previous, childStatement) => {
            if (!childStatement) {
                return previous;
            }
            if (childStatement.type === "namespace") {
                return previous.concat(namespaceToTsStatement(childStatement));
            }
            if (childStatement.type === "typeAlias") {
                return previous.concat(typeAliasToStatement(childStatement));
            }
            return previous.concat(interfaceToStatement(childStatement));
        }, []);
        return factory.Namespace.addStatements({
            node: node.value,
            statements,
        });
    };
    const dump = (filename) => {
        fs.writeFileSync(filename, yaml.dump(Masking.maskValue(state)), { encoding: "utf-8" });
    };
    const dumpOperationState = (filename) => {
        fs.writeFileSync(filename, JSON.stringify(state.operations, null, 2), { encoding: "utf-8" });
    };
    const addAdditionalStatement = (statements) => {
        state.additionalStatements = state.additionalStatements.concat(statements);
    };
    const getRootStatements = () => {
        const statements = Def.componentNames.reduce((statements, componentName) => {
            const component = state.components[Def.generateKey("namespace", componentName)];
            if (!component) {
                return statements;
            }
            if (component.type === "interface") {
                return statements.concat(interfaceToStatement(component));
            }
            if (component.type === "namespace") {
                return statements.concat(namespaceToTsStatement(component));
            }
            if (component.type === "typeAlias") {
                return statements.concat(typeAliasToStatement(component));
            }
            return statements;
        }, []);
        return statements.concat(state.additionalStatements);
    };
    const updateOperationState = (httpMethod, requestUri, operationId, newOperationState) => {
        let operationState = state.operations[operationId];
        if (operationState) {
            operationState = { ...operationState, ...newOperationState };
        }
        else {
            operationState = State.createDefaultOperationState(httpMethod, requestUri, newOperationState);
        }
        state.operations[operationId] = operationState;
    };
    const getOperationState = (operationId) => {
        const operationState = state.operations[operationId];
        if (operationState) {
            return operationState;
        }
        dumpOperationState("debug/error-getOperationState.json");
        throw new Error(`Not found ${operationId}`);
    };
    const getNoReferenceOperationState = () => {
        return Operation.create(state.document);
    };
    return {
        hasStatement,
        addStatement,
        getStatement,
        getRootStatements,
        addComponent,
        dump,
        getNoReferenceOperationState,
        updateOperationState,
        getOperationState,
        addAdditionalStatement,
        dumpOperationState,
        getPathItem,
    };
};
//# sourceMappingURL=Store.js.map